import React from "react";

import { withApollo } from "react-apollo";
import { Field, Form } from "react-final-form";

import FormAsideAdminUser from "../../../components/FormAsideAdminUser";
import {
  PropertyAsideBoomin,
  PropertyCardItemTitle,
  PropertyCardItemValue,
  PropertyContent,
  PropertyItemBoomin,
  PropertyItemPublish,
  PublicationBlocked,
} from "../../../ui/atoms/Property";
import { TitleError } from "../../../ui/atoms/TitleError";
import { TitleSuccess } from "../../../ui/atoms/TitleSuccess";
import { UserFormRow } from "../../../ui/atoms/UserForm";
import { CheckboxToggle } from "../../../ui/molecules/CheckboxToggle";
import CopyText from "../../../ui/molecules/CopyToClipboard";
import { ModalAlert, ModalConfirm } from "../../../ui/molecules/Modal";
import { PropertyBoomin } from "../graphql/queries/__generated__/PropertyBoomin";
import VerifyContext from "../PropertyEditContext";
import { usePublish } from "./use-publish";

type Props = {
  propertyId: string;
  client?: any;
  boomin: PropertyBoomin;
  version_id: string;
};

const PublishBoomin = ({ boomin, version_id }: Props) => {
  const {
    errorText,
    modalMode,
    onClearMessage,
    onSubmit,
    publishing,
    setModalMode,
    successText,
    unpublishing,
  } = usePublish(version_id, "boomin");

  return (
    <>
      {errorText || successText ? (
        <ModalAlert
          onCancel={onClearMessage}
          title={
            errorText != null ? (
              <TitleError>Error</TitleError>
            ) : (
              <TitleSuccess>Message</TitleSuccess>
            )
          }
        >
          <b>{errorText || successText}</b>
        </ModalAlert>
      ) : null}
      <VerifyContext.Consumer>
        {({ publishedInBoomin, publishedInMashroom }) => (
          <Form
            initialValues={{
              publish_property_boomin: publishedInBoomin,
            }}
            onSubmit={(fn) => fn}
            render={({ form }) => {
              return (
                <>
                  {(modalMode === "confirm" || publishing) && (
                    <ModalConfirm
                      disabled={publishing}
                      loading={publishing}
                      onCancel={() => {
                        form.reset();
                        setModalMode(undefined);
                      }}
                      onConfirm={() => {
                        onSubmit(form, "publish");
                        setModalMode(undefined);
                      }}
                    >
                      <span>
                        Please confirm
                        <br />
                        <b>publish</b> property on Boomin?
                      </span>
                    </ModalConfirm>
                  )}
                  {modalMode === "delete" ||
                    (unpublishing && (
                      <ModalConfirm
                        disabled={unpublishing}
                        loading={unpublishing}
                        onCancel={() => {
                          form.reset();
                          setModalMode(undefined);
                        }}
                        onConfirm={() => {
                          onSubmit(form, "unpublish");
                          setModalMode(undefined);
                        }}
                      >
                        <span>
                          Are you sure you really want to
                          <br />
                          <b>unpublish</b> property from Boomin?
                        </span>
                      </ModalConfirm>
                    ))}
                  <PropertyItemBoomin>
                    <PropertyContent>
                      <PropertyItemPublish>
                        <h3>Boomin</h3>
                        {publishedInBoomin && (
                          <>
                            <PropertyCardItemTitle>
                              Property ID
                            </PropertyCardItemTitle>
                            <PropertyCardItemValue>
                              {boomin.id}
                              <CopyText
                                text="Copy Boomin ID"
                                textForCopy={boomin.id}
                              />
                            </PropertyCardItemValue>
                            <br />
                            <PropertyCardItemTitle>
                              Property Link
                            </PropertyCardItemTitle>
                            <PropertyCardItemValue>
                              <a
                                className="link"
                                href={boomin.url}
                                rel="noopener noreferrer"
                                target="_blank"
                                title={`Open in new tab ${boomin.url}`}
                              >
                                {`${boomin.url.substring(
                                  0,
                                  30
                                )} . . . ${boomin.url.substr(-10)}`}
                              </a>
                              <CopyText
                                text="Copy link"
                                textForCopy={boomin.url}
                              />
                            </PropertyCardItemValue>
                          </>
                        )}
                        {!publishedInBoomin && publishedInMashroom && (
                          <h4>
                            Something went wrong. Please, try again later.
                          </h4>
                        )}
                        {!publishedInBoomin && !publishedInMashroom && (
                          <h4>
                            Property will not be published on Boomin until it is
                            published on Mashroom
                          </h4>
                        )}
                      </PropertyItemPublish>
                    </PropertyContent>
                    <PropertyAsideBoomin>
                      <UserFormRow>
                        {!publishedInBoomin && !publishedInMashroom ? (
                          <PublicationBlocked>
                            <Field
                              component={CheckboxToggle}
                              disabled={
                                publishedInBoomin ||
                                (!publishedInBoomin && !publishedInMashroom)
                              }
                              labelText="Published"
                              name="publish_property_boomin"
                              onChange={(event) => {
                                form.change(
                                  "publish_property_boomin",
                                  event.target.checked
                                );
                                setModalMode("confirm");
                              }}
                              type="checkbox"
                            />
                          </PublicationBlocked>
                        ) : (
                          <Field
                            component={CheckboxToggle}
                            disabled={
                              !publishedInBoomin && !publishedInMashroom
                            }
                            labelText="Published"
                            name="publish_property_boomin"
                            onChange={(event) => {
                              form.change(
                                "publish_property_boomin",
                                event.target.checked
                              );
                              setModalMode(
                                event.target.checked ? "confirm" : "delete"
                              );
                            }}
                            type="checkbox"
                          />
                        )}
                      </UserFormRow>
                      {publishedInBoomin && (
                        <FormAsideAdminUser
                          date={boomin.created_at as string}
                          id={boomin.published_by as string}
                        />
                      )}
                    </PropertyAsideBoomin>
                  </PropertyItemBoomin>
                </>
              );
            }}
          />
        )}
      </VerifyContext.Consumer>
    </>
  );
};

export default withApollo<Props>(PublishBoomin);
