import React, { ReactNode, useMemo, useState } from "react";

import { DocumentNode } from "graphql";
import { noop } from "lodash";
import { useApolloClient, withApollo } from "react-apollo";
import { Field, Form } from "react-final-form";

import FormAsideAdminUser from "../../../components/FormAsideAdminUser";
import { property_api_PropertyStatusEnum } from "../../../globalTypes";
import {
  MessagePublished,
  MessageSomethingWrong,
} from "../../../ui/atoms/Message";
import {
  PropertyAsideMashroom,
  PropertyCardItemTitle,
  PropertyCardItemValue,
  PropertyContent,
  PropertyItemMashroom,
  PropertyItemPublish,
  PublicationBlocked,
} from "../../../ui/atoms/Property";
import { UserFormRow } from "../../../ui/atoms/UserForm";
import { CheckboxToggle } from "../../../ui/molecules/CheckboxToggle";
import CopyText from "../../../ui/molecules/CopyToClipboard";
import { ModalAlert, ModalConfirm } from "../../../ui/molecules/Modal";
import PUBLISH_VERSION from "../graphql/mutations/PublishVersion.graphql";
import UNPUBLISH_VERSION from "../graphql/mutations/UnpublishVersion.graphql";
import VerifyContext from "../PropertyEditContext";
import { PublishRightmove } from ".";

type Props = {
  propertyId: string;
  client?: any;
  publishedDate?: string;
  publishedUser?: string;
  propertyStatus: property_api_PropertyStatusEnum;
  version_id: string;
  publishedInBoomin: boolean | undefined;
  publishedInMashroom: boolean | undefined;
  publishedInRightmove: boolean | undefined;
  publishedInZoopla: boolean | undefined;
};

const messages = {
  publishProperty: {
    success: "Property was published!",
  },
  unpublishProperty: {
    success: "Property was unpublished.",
  },
};

const messagesUnpublish = {
  other_status:
    "Mashroom: You can't publish the property if it's not in the verifying status",
  not_verifyed: "Mashroom: Before publication you must confirm -",
};

const PublishMashroom = ({
  publishedInBoomin,
  publishedInRightmove,
  publishedInZoopla,
  propertyId,
  publishedUser,
  publishedDate,
  propertyStatus,
  version_id,
}: Props) => {
  const client = useApolloClient();
  const [showConfirm, setConfirm] = useState(false);
  const [alert, setAlert] = useState<ReactNode>();
  const propertyUrl = `${process.env.REACT_APP_VHOST}/property/${propertyId}`;

  const isVerifying = useMemo(
    () =>
      propertyStatus === property_api_PropertyStatusEnum.VERIFYING ||
      propertyStatus === property_api_PropertyStatusEnum.MISSING ||
      propertyStatus === property_api_PropertyStatusEnum.AMENDMENT,
    [propertyStatus]
  );

  return (
    <VerifyContext.Consumer>
      {({
        addressVerified,
        descriptionVerified,
        documentsVerified,
        feeVerified,
        photosVerified,
        publishedInMashroom,
        setPublishedInMashroom,
        refetchProperty,
      }) => (
        <Form
          initialValues={{
            publish_property: publishedInMashroom,
          }}
          onSubmit={noop}
          render={({ submitting, form, values }) => {
            const handleFormSubmit = () => {
              let mutation: DocumentNode,
                resultKey: string,
                mutationVariable: any;
              if (values.publish_property) {
                mutation = PUBLISH_VERSION;
                resultKey = "publishProperty";
                mutationVariable = {};
              } else {
                mutation = UNPUBLISH_VERSION;
                resultKey = "unpublishProperty";
                mutationVariable = true;

                const publishedOn = [
                  { value: publishedInZoopla, label: "Zoopla" },
                  { value: publishedInBoomin, label: "Boomin" },
                  { value: PublishRightmove, label: "Rightmove" },
                ]
                  .filter(({ value }) => !!value)
                  .map(({ label }) => label);

                if (
                  publishedInRightmove ||
                  publishedInZoopla ||
                  publishedInBoomin
                ) {
                  form.reset();
                  return setAlert(
                    `Property is published on ${publishedOn.join(
                      " and "
                    )}. Unpublish it ${
                      publishedOn.length > 1 ? "on those platforms" : "there"
                    } before unpublishing on Mashroom`
                  );
                }
              }

              return client
                .mutate({
                  mutation,
                  variables: {
                    version_id,
                    mashroom: mutationVariable,
                  },
                })
                .then((data) => {
                  const result =
                    data?.data?.admin?.publishVersion?.version?.id ??
                    data?.data?.admin?.unpublishVersion?.version?.id;
                  const errors = data?.errors ?? [];

                  if (errors.length) {
                    setAlert(<>{errors.map((e) => e.message).join(", ")}</>);
                    form.reset();
                    return;
                  }

                  let textMessage: React.ReactNode;

                  if (result) {
                    void setPublishedInMashroom?.(
                      resultKey === "publishProperty"
                    );
                    textMessage = (
                      <MessagePublished msg={messages[resultKey].success} />
                    );
                  } else {
                    textMessage = <MessageSomethingWrong />;
                    form.reset();
                  }
                  void refetchProperty?.();

                  setAlert(textMessage);
                });
            };
            const VerifiedTitles = [
              { k: addressVerified, v: "Address" },
              { k: descriptionVerified, v: "Description" },
              { k: documentsVerified, v: "Documents" },
              { k: photosVerified, v: "Photos" },
              { k: feeVerified, v: "Listing Fee" },
            ]
              .map((i) => (i.k ? null : i.v))
              .filter(Boolean);

            return (
              <>
                {alert && (
                  <ModalAlert
                    onCancel={() => {
                      setAlert("");
                    }}
                  >
                    <b>{alert}</b>
                  </ModalAlert>
                )}
                {showConfirm && (
                  <ModalConfirm
                    onCancel={() => {
                      form.reset();
                      setConfirm(false);
                    }}
                    onConfirm={() => {
                      handleFormSubmit();
                      setConfirm(false);
                    }}
                  >
                    <span>
                      Are you sure you really want to
                      <br />
                      <b>
                        {publishedInMashroom ? "unpublish" : "publish"}
                      </b>{" "}
                      property?
                    </span>
                  </ModalConfirm>
                )}
                <PropertyItemMashroom>
                  <PropertyContent>
                    <PropertyItemPublish>
                      {(!isVerifying ||
                        !addressVerified ||
                        !descriptionVerified ||
                        !photosVerified ||
                        !feeVerified ||
                        !documentsVerified) &&
                      !publishedInMashroom ? (
                        <>
                          {isVerifying ? (
                            <h3>
                              {messagesUnpublish.not_verifyed}{" "}
                              {VerifiedTitles.join(", ")}
                            </h3>
                          ) : (
                            <>
                              <h3>{messagesUnpublish.other_status}</h3>
                              <PropertyCardItemTitle>
                                Property STATUS
                              </PropertyCardItemTitle>
                              <PropertyCardItemValue>
                                <b className="text-danger">{propertyStatus}</b>
                              </PropertyCardItemValue>
                              <br />
                            </>
                          )}
                        </>
                      ) : (
                        <>
                          <h3>Mashroom</h3>
                          <PropertyCardItemTitle>
                            Property ID
                          </PropertyCardItemTitle>
                          <PropertyCardItemValue>
                            {propertyId}
                            <CopyText
                              text="Copy property ID"
                              textForCopy={propertyId}
                            />
                          </PropertyCardItemValue>
                          <br />
                        </>
                      )}
                      <PropertyCardItemTitle>
                        Property Link
                      </PropertyCardItemTitle>
                      <PropertyCardItemValue>
                        <a
                          className="link"
                          href={propertyUrl}
                          rel="noopener noreferrer"
                          target="_blank"
                          title={`Open in new tab ${propertyUrl}`}
                        >
                          {`${propertyUrl.substring(
                            0,
                            30
                          )} . . . ${propertyUrl.substr(-10)}`}
                        </a>
                        <CopyText text="Copy link" textForCopy={propertyUrl} />
                      </PropertyCardItemValue>
                    </PropertyItemPublish>
                  </PropertyContent>
                  <PropertyAsideMashroom>
                    <UserFormRow>
                      {(propertyStatus !==
                        property_api_PropertyStatusEnum.VERIFYING ||
                        !addressVerified ||
                        !descriptionVerified ||
                        !photosVerified ||
                        !feeVerified ||
                        !documentsVerified) &&
                      !publishedInMashroom ? (
                        <PublicationBlocked>
                          <Field
                            component={CheckboxToggle}
                            disabled={
                              submitting ||
                              !addressVerified ||
                              !descriptionVerified ||
                              !photosVerified ||
                              !feeVerified ||
                              !documentsVerified
                            }
                            labelText="Published"
                            name="publish_property"
                            onChange={(event) => {
                              setConfirm(true);
                              form.change(
                                "publish_property",
                                event.target.checked
                              );
                            }}
                            type="checkbox"
                          />
                        </PublicationBlocked>
                      ) : (
                        <Field
                          component={CheckboxToggle}
                          disabled={submitting}
                          labelText="Published"
                          name="publish_property"
                          onChange={(event) => {
                            setConfirm(true);
                            form.change(
                              "publish_property",
                              event.target.checked
                            );
                          }}
                          type="checkbox"
                        />
                      )}
                    </UserFormRow>
                    {publishedInMashroom && (
                      <FormAsideAdminUser
                        date={publishedDate as string}
                        id={publishedUser as string}
                      />
                    )}
                  </PropertyAsideMashroom>
                </PropertyItemMashroom>
              </>
            );
          }}
        />
      )}
    </VerifyContext.Consumer>
  );
};

export default withApollo<Props>(PublishMashroom);
