import { useCallback, useState } from "react";

import { FormApi } from "final-form";
import { GraphQLError } from "graphql";
import { useMutation } from "react-apollo";

import {
  PublishVersion,
  PublishVersionVariables,
} from "../graphql/mutations/__generated__/PublishVersion";
import {
  UnpublishVersion,
  UnpublishVersionVariables,
} from "../graphql/mutations/__generated__/UnpublishVersion";
import PUBLISH_VERSION from "../graphql/mutations/PublishVersion.graphql";
import UNPUBLISH_VERSION from "../graphql/mutations/UnpublishVersion.graphql";

export const usePublish = (
  version_id: string,
  serviceName: "boomin" | "zoopla" | "rightmove"
) => {
  const [modalMode, setModalMode] = useState<
    "confirm" | "delete" | undefined
  >();
  const [errorText, setErrorText] = useState<string | undefined>();
  const [successText, setSuccessText] = useState<string | undefined>();
  const [publishVersion, { loading: publishing }] = useMutation<
    PublishVersion,
    PublishVersionVariables
  >(PUBLISH_VERSION);
  const [unpublishVersion, { loading: unpublishing }] = useMutation<
    UnpublishVersion,
    UnpublishVersionVariables
  >(UNPUBLISH_VERSION);

  const update = useCallback(
    async (
      type: "publish" | "unpublish",
      vars: PublishVersionVariables | UnpublishVersionVariables
    ) => {
      const params = {
        awaitRefetchQueries: true,
        refetchQueries: ["GetProperty"],
      };

      if (type === "publish") {
        const response = await publishVersion({
          ...params,
          variables: vars as PublishVersionVariables,
        });

        return response;
      }

      const response = await unpublishVersion({
        ...params,
        variables: vars as UnpublishVersionVariables,
      });

      return response;
    },
    [publishVersion, unpublishVersion]
  );

  const setErrors = useCallback((errors: GraphQLError[]) => {
    setErrorText(
      errors
        .map((err) => err.message ?? "Something went wrong, try again later")
        .join(", ")
    );
  }, []);

  const handleSubmit = useCallback(
    async (
      form: FormApi<Record<string, any>, Partial<Record<string, any>>>,
      type: "publish" | "unpublish"
    ) => {
      const response = await update(type, {
        version_id,
        [serviceName]: type === "publish" ? {} : true,
      });

      if ((response?.errors ?? []).length > 0) {
        setErrors(response?.errors ?? []);
        form.reset();
      } else {
        setSuccessText(
          type === "publish"
            ? "Property was published succesfully."
            : "Property was unpublished."
        );
      }
    },
    [update, setErrors, serviceName, version_id]
  );

  const handleClearMessage = useCallback(() => {
    setSuccessText(undefined);
    setErrorText(undefined);
  }, []);

  return {
    errorText,
    modalMode,
    onClearMessage: handleClearMessage,
    onSubmit: handleSubmit,
    publishing,
    setModalMode,
    successText,
    unpublishing,
  };
};
