import React from "react";

import styled from "@emotion/styled";
import { sentenceCase } from "change-case";
import { format } from "date-fns";
import { Query } from "react-apollo";
import { Link } from "react-router-dom";
import { oc } from "ts-optchain";

import {
  PropertyBody,
  PropertyCardItemTitle,
  PropertyContent,
} from "../../../ui/atoms/Property";
import {
  Table,
  Tbody,
  TdInner,
  Thead,
  ThInner,
  Tr,
  TrInner,
} from "../../../ui/atoms/Table";
import {
  UserFormItem,
  UserFormRowFlex,
  UserFormValue,
} from "../../../ui/atoms/UserForm";
import CopyText from "../../../ui/molecules/CopyToClipboard";
import Pagebar from "../../../ui/molecules/Pagebar";
import Pagination from "../../../ui/molecules/Pagination";
import { feeStatusFilter } from "../filters";
import { GetProperty_admin_Property } from "../graphql/queries/__generated__/GetProperty";
import {
  GetUser,
  GetUserVariables,
} from "../graphql/queries/__generated__/GetUser";
import GET_USER from "../graphql/queries/GetUser.graphql";
import VerifyContext from "../PropertyEditContext";
import PropertyVerification from "../PropertyVerification";

type Props = {
  property: GetProperty_admin_Property;
  fees: any;
};

const Fees = (props: Props) => {
  const perPage: number = 1;
  const fees = props.property.listing_fee;

  if (!fees) {
    return null;
  }

  return (
    <VerifyContext.Consumer>
      {({ feeVerified }) => (
        <PropertyBody>
          <PropertyContent>
            {fees.length > 0 ? (
              <Pagination perPage={perPage}>
                {({ page, setPage }) => (
                  <>
                    {fees.map((itemFee, index) => {
                      const range = page * perPage;

                      if (index + 1 > range || index + 1 <= range - perPage) {
                        return null;
                      }

                      return (
                        <React.Fragment key={itemFee.id}>
                          <UserFormRowFlex>
                            <UserFormItem>
                              <PropertyCardItemTitle>
                                Payment Status
                              </PropertyCardItemTitle>
                              <UserFormValue>
                                <b>{feeStatusFilter(itemFee.status)}</b>
                              </UserFormValue>
                            </UserFormItem>
                            <UserFormItem>
                              <PropertyCardItemTitle>
                                Payment ID
                              </PropertyCardItemTitle>
                              <UserFormValue>
                                {`. . . ${itemFee.id.substr(-6)}`}

                                <CopyText
                                  text="Copy ID"
                                  textForCopy={itemFee.id}
                                />
                              </UserFormValue>
                            </UserFormItem>
                          </UserFormRowFlex>
                          <UserFormRowFlex>
                            <UserFormItem>
                              <PropertyCardItemTitle>
                                Created
                              </PropertyCardItemTitle>
                              <UserFormValue>
                                {itemFee.created_at
                                  ? format(
                                      new Date(itemFee.created_at),
                                      "dd.MM.yyyy HH:mm:ss"
                                    )
                                  : "- - -"}
                              </UserFormValue>
                            </UserFormItem>
                            <UserFormItem>
                              <PropertyCardItemTitle>
                                Created by
                              </PropertyCardItemTitle>
                              <Query<GetUser, GetUserVariables>
                                query={GET_USER}
                                variables={{ id: itemFee.user_id }}
                              >
                                {({ loading, error, data }) => {
                                  if (loading) {
                                    return <>Loading...</>;
                                  } else if (error) {
                                    return <>error</>;
                                  }

                                  const user = oc(data).admin.User();

                                  if (!user) {
                                    return null;
                                  }

                                  return (
                                    <UserFormValue>
                                      {
                                        <Link
                                          className="link"
                                          to={`/users/${user.id}/view`}
                                        >
                                          <>
                                            {oc(user).details.first_name("")}
                                            &nbsp;
                                            {oc(user).details.last_name("")}
                                          </>
                                        </Link>
                                      }
                                    </UserFormValue>
                                  );
                                }}
                              </Query>
                            </UserFormItem>
                            <UserFormItem>
                              <PropertyCardItemTitle>
                                Updated
                              </PropertyCardItemTitle>
                              <UserFormValue>
                                {itemFee.updated_at
                                  ? format(
                                      new Date(itemFee.updated_at),
                                      "dd.MM.yyyy HH:mm:ss"
                                    )
                                  : "- - -"}
                              </UserFormValue>
                            </UserFormItem>
                          </UserFormRowFlex>
                          <br />
                          <br />
                          <Table>
                            <Thead>
                              <Tr>
                                <ThInner>Code</ThInner>
                                <ThInner>Type</ThInner>
                                <ThInner>Description</ThInner>
                                <ThInner>Amount</ThInner>
                              </Tr>
                            </Thead>
                            <Tbody>
                              {(itemFee.items ? itemFee.items : []).map(
                                (item) => (
                                  <TrInner key={item.id as string}>
                                    <TdInner>{item.code}</TdInner>
                                    <TdInner>
                                      <b>
                                        {sentenceCase(
                                          item.type ? item.type : ""
                                        )}
                                      </b>
                                    </TdInner>
                                    <TdInner>{item.description}</TdInner>
                                    <TdInner>
                                      <b>&pound; {item.price}</b>
                                    </TdInner>
                                  </TrInner>
                                )
                              )}
                            </Tbody>
                          </Table>

                          <Pagebar
                            changePage={setPage}
                            curPage={page}
                            showNPages={3}
                            total={fees.length}
                          />
                        </React.Fragment>
                      );
                    })}
                  </>
                )}
              </Pagination>
            ) : (
              <NotFound>Listing fees not found</NotFound>
            )}
          </PropertyContent>

          <PropertyVerification
            adminVerify={props.property.admin_verify}
            isVerified={!!feeVerified}
            property={props.property}
            verifiedLabel="Verified"
            verifiedName="admin_verify.fee.verified"
            verifyKey="fee"
          />
        </PropertyBody>
      )}
    </VerifyContext.Consumer>
  );
};

const NotFound = styled.div`
  text-align: center;
  font-size: 18px;
  font-weight: 600;
`;

export default Fees;
