import React, { FC, useCallback, useMemo, useState } from "react";

import { sentenceCase } from "change-case";
import Cookies from "js-cookie";
import { useMutation, useQuery } from "react-apollo";
import { Link, useParams } from "react-router-dom";

import {
  common_api_generic_query_QueryFilterOperatorEnum,
  invoice_api_TransactionReconciliationEnum,
} from "../../../../globalTypes";
import { SecondaryButton } from "../../../../ui/atoms/Button";
import {
  ColumnNarrow,
  ColumnTitle,
  ColumnValue,
  ColumnWide,
  PaymentBody,
  PaymentConfirmed,
  PaymentCreatedAt,
  PaymentDescription,
  PaymentHead,
  PaymentRow,
  PaymentsGroup,
  PaymentsGroupHead,
} from "../../../../ui/atoms/Payment";
import Spinner from "../../../../ui/atoms/Spinner";
import SystemError from "../../../../ui/atoms/SystemError";
import { ModalAlert } from "../../../../ui/molecules/Modal";
import { formatDateTime } from "../../../../util/date";
import { formatPrice } from "../../../../util/price";
import {
  PaymentsCreatePayout,
  PaymentsCreatePayoutVariables,
} from "../../graphql/mutations/__generated__/PaymentsCreatePayout";
import CREATE_PAYOUT from "../../graphql/mutations/PaymentsCreatePayout.graphql";
import {
  PaymentSearchProperty,
  PaymentSearchPropertyVariables,
} from "../../graphql/queries/__generated__/PaymentSearchProperty";
import {
  PaymentsGetTransaction,
  PaymentsGetTransactionVariables,
} from "../../graphql/queries/__generated__/PaymentsGetTransaction";
import SEARCH_PROPERTY_BY_IVR from "../../graphql/queries/PaymentSearchProperty.graphql";
import GET_TRANSACTION from "../../graphql/queries/PaymentsGetTransaction.graphql";
import BankDetails from "../BankDetails";
import { Matches } from "./MatchesTable";
import { MatchTransactionForm } from "./MatchTransactionForm";

// import GET_ALL_SCHEDULES from "../graphql/queries/PaymentsGetAllInvoices.graphql";

const TransactionDetails: FC = () => {
  const [alertMessage, setAlertMessage] = useState<string | null>(null);
  const { transactionId } = useParams();

  const [createPayout, { loading: payoutLoading }] = useMutation<
    PaymentsCreatePayout,
    PaymentsCreatePayoutVariables
  >(CREATE_PAYOUT, {
    refetchQueries: ["PaymentsGetAllTransactions", "PaymentSearchProperty"],
  });

  const { data, loading, error } = useQuery<
    PaymentsGetTransaction,
    PaymentsGetTransactionVariables
  >(GET_TRANSACTION, {
    variables: {
      transaction_id: transactionId,
    },
    skip: transactionId == null,
  });

  const transactionData = data?.admin?.Transaction;
  const ivr = transactionData?.ivr;
  const payoutValue = (transactionData?.scheduled_records?.scheduledRecords ??
    [])[0]?.record.type.income?.rent?.payouts[0].value;
  const invoiceId = payoutValue?.invoice_id;
  const payoutStatus = payoutValue?.payout_status;
  const amount = payoutValue?.amount;
  const offerId = transactionData?.offer_id;

  const {
    data: propertyData,
    loading: propertyLoading,
    error: propertyError,
  } = useQuery<PaymentSearchProperty, PaymentSearchPropertyVariables>(
    SEARCH_PROPERTY_BY_IVR,
    {
      variables: {
        filter: [
          {
            fields: ["ivr"],
            value: ivr ? [ivr] : [""],
            operator: common_api_generic_query_QueryFilterOperatorEnum.EQUALS,
          },
        ],
      },
      skip: !ivr,
    }
  );

  const property = useMemo(
    () => propertyData?.admin?.allProperties?.data?.[0],
    [propertyData]
  );

  const createPayoutForPayment = useCallback(() => {
    if (offerId && invoiceId) {
      createPayout({
        variables: {
          offer: offerId,
          invoice_id: invoiceId,
        },
      }).then((res) => {
        const resPayout = res?.data?.admin?.createPayout;
        const tokenio = resPayout?.tokenio;

        if (resPayout && tokenio != null && window != null) {
          Cookies.set("csrf_tokenio", tokenio.csrf);
          window.location.assign(tokenio.url);
        } else if (res.errors != null) {
          setAlertMessage(res.errors.map((item) => item.message).join(", "));
        }
      });
    }
  }, [createPayout, invoiceId, offerId]);

  if (loading) {
    return <Spinner />;
  }

  if (error) {
    return <>{error.message}</>;
  }

  if (!transactionData) {
    return null;
  }

  const isFullyMatched =
    transactionData.reconciliation ===
    invoice_api_TransactionReconciliationEnum.TRANSACTION_RECONCILIATION_FULL;

  return (
    <>
      <PaymentsGroup>
        <PaymentsGroupHead>
          <Link className="link-button" to="/statement">
            <SecondaryButton>Back</SecondaryButton>
          </Link>
          <h2>Transaction</h2>
        </PaymentsGroupHead>

        <PaymentConfirmed>
          <PaymentHead>
            <PaymentDescription>
              Transaction ID <b>{transactionData.id}</b>
            </PaymentDescription>
            <PaymentDescription>
              IVR <b>{ivr ?? "Unknown"}</b>
            </PaymentDescription>
            <PaymentCreatedAt>
              {formatDateTime(transactionData.created)}
            </PaymentCreatedAt>
          </PaymentHead>
          {!isFullyMatched && (
            <MatchTransactionForm
              transaction_id={transactionId}
              transactionMatches={transactionData.matches}
            />
          )}
          <PaymentBody>
            {propertyLoading ? (
              <Spinner size="sm" />
            ) : propertyError ? (
              <SystemError error={propertyError} />
            ) : property ? (
              <PaymentRow>
                <ColumnNarrow>
                  <ColumnTitle>Address</ColumnTitle>
                  <ColumnValue>{property.location?.address}</ColumnValue>
                </ColumnNarrow>
                <ColumnNarrow>
                  <ColumnTitle>Postcode</ColumnTitle>
                  <ColumnValue>
                    {property.location?.postcode ?? "- - -"}
                  </ColumnValue>
                </ColumnNarrow>
                <ColumnNarrow>
                  <ColumnWithLink
                    href={`/users/${property.people?.landlord?.id}/view`}
                    title="Landlord Name"
                  >
                    {[
                      property.people?.landlord?.details?.first_name ?? "- - -",
                      property.people?.landlord?.details?.last_name ?? "- - -",
                    ].join(" ")}
                  </ColumnWithLink>
                </ColumnNarrow>
              </PaymentRow>
            ) : (
              <PaymentRow>
                <ColumnWide>
                  <SystemError error="Can't find the property that is associated with the transaction" />
                </ColumnWide>
                <ColumnNarrow>
                  <OfferColumn offerId={offerId} />
                </ColumnNarrow>
              </PaymentRow>
            )}

            {property ? (
              <PaymentRow>
                <ColumnWide>
                  <OfferColumn offerId={offerId} />
                </ColumnWide>
                <ColumnNarrow>
                  <ColumnWithLink
                    href={`/listing/${property.id}/edit`}
                    title="Property ID"
                  >
                    {property?.id}
                  </ColumnWithLink>
                </ColumnNarrow>
              </PaymentRow>
            ) : null}

            <div style={{ margin: "0 0 32px", borderTop: "solid 1px #eee" }} />

            <PaymentRow>
              <ColumnWide>
                <ColumnTitle>Description</ColumnTitle>
                <ColumnValue>{transactionData.description}</ColumnValue>
              </ColumnWide>
              <ColumnNarrow>
                <ColumnTitle>Amount</ColumnTitle>
                <ColumnValue>{formatPrice(transactionData.amount)}</ColumnValue>
              </ColumnNarrow>
            </PaymentRow>
            <PaymentRow>
              <ColumnNarrow>
                <ColumnTitle>Status</ColumnTitle>
                <ColumnValue>
                  {sentenceCase(
                    transactionData.status.replace("TRANSACTION_STATUS_", "")
                  )}
                </ColumnValue>
              </ColumnNarrow>
              <ColumnNarrow>
                <ColumnTitle>Type</ColumnTitle>
                <ColumnValue>
                  {sentenceCase(
                    transactionData.type.replace("TRANSACTION_TYPE_", "")
                  )}
                </ColumnValue>
              </ColumnNarrow>
              <ColumnNarrow>
                <ColumnTitle>Reconciliation</ColumnTitle>
                <ColumnValue>
                  {sentenceCase(
                    transactionData.reconciliation.replace(
                      "TRANSACTION_RECONCILIATION_",
                      ""
                    )
                  )}
                </ColumnValue>
              </ColumnNarrow>
            </PaymentRow>

            <ColumnTitle>Matches</ColumnTitle>
            {(transactionData.matches ?? []).length > 0 ? (
              <>
                <Matches
                  transactionId={transactionId}
                  matches={transactionData.matches}
                  isFullyMatched={isFullyMatched}
                  amount={amount}
                />
                <BankDetails
                  landlordId={property?.people?.landlord?.id}
                  loading={payoutLoading}
                  onPayout={() => createPayoutForPayment()}
                  payoutStatus={payoutStatus}
                  sum={formatPrice(transactionData.amount)}
                  visible={Boolean(offerId && invoiceId)}
                />
              </>
            ) : (
              <div style={{ marginBottom: "24px" }}>No matches</div>
            )}
          </PaymentBody>
        </PaymentConfirmed>
      </PaymentsGroup>

      {alertMessage && (
        <ModalAlert
          onCancel={() => {
            setAlertMessage(null);
          }}
        >
          <b>{alertMessage}</b>
        </ModalAlert>
      )}
    </>
  );
};

export default TransactionDetails;

const ColumnWithLink: FC<{ title: string; href: string }> = ({
  title,
  href,
  children,
}) => (
  <>
    <ColumnTitle>{title}</ColumnTitle>
    <ColumnValue>
      <a className="link" href={href} rel="noopener noreferrer" target="_blank">
        {children}
      </a>
    </ColumnValue>
  </>
);

const OfferColumn: FC<{ offerId: string | null | undefined }> = ({
  offerId,
}) => (
  <>
    {offerId ? (
      <ColumnWithLink href={`#/offers/${offerId}/edit`} title="Offer ID">
        {offerId}
      </ColumnWithLink>
    ) : (
      <>
        <ColumnTitle>Offer ID</ColumnTitle>
        <ColumnValue>Transaction is not related to offer</ColumnValue>
      </>
    )}
  </>
);
