import React, { FC, useCallback, useEffect, useState } from "react";

import {
  CardNumberElement,
  useElements,
  useStripe,
} from "@stripe/react-stripe-js";
import { StripeCardElementChangeEvent } from "@stripe/stripe-js";
import { Field, Form } from "react-final-form";

import Button from "../../../components/Button";
import Spinner from "../../../components/Spinner";
import StripePayment from "../../../components/StripePayment";
import { PaymentDetails } from "../../../models/payment";
import {
  CardContainer,
  CardHeader,
  CardTitle,
  ErrorContainer,
} from "../../../styles";
import { colors } from "../../../theme";
import {
  PaymentDetailsContainer,
  PaymentFormBody,
  SpinnerContainer,
} from "./styles";

type PaymentFormProps = {
  payDetails: PaymentDetails;
  paymentConfirmToken: string;
  onPaymentSubmit: () => void;
  onPaymentConfirmed: (intentId: string) => void;
  transactionInProgress: boolean;
};

const PaymentForm: FC<PaymentFormProps> = ({
  payDetails,
  paymentConfirmToken,
  onPaymentSubmit,
  onPaymentConfirmed,
  transactionInProgress,
}) => {
  const [formElementsComplete, setFormElementsComplete] = useState<
    Map<string, boolean>
  >(new Map());
  const [formComplete, setFormComplete] = useState<boolean>(false);
  const [error, setError] = useState<string>("");

  const stripe = useStripe();
  const elements = useElements();
  const cardNumEl = elements?.getElement(CardNumberElement);

  const isFormCompleted = (formCompleteMap: Map<string, boolean>): boolean =>
    (formCompleteMap.get("cardNumber") &&
      formCompleteMap.get("cardExpiry") &&
      formCompleteMap.get("cardCvc")) ||
    false;

  const validate = (value: StripeCardElementChangeEvent): string => {
    if (!value) {
      return "";
    }

    formElementsComplete.set(String(value.elementType), value.complete);
    setFormComplete(isFormCompleted(formElementsComplete));
    setFormElementsComplete(new Map(formElementsComplete));

    if (value?.empty || value.error?.message) {
      return `${String(value?.elementType)} is required`;
    }
    return "";
  };

  const confirmCardPayment = useCallback(
    async (stripeToken): Promise<string> => {
      if (!stripeToken || !cardNumEl) {
        throw Error("stripeToken or cardNumEl is undefined");
      }

      const confirmResult = await stripe!.confirmCardPayment(stripeToken, {
        payment_method: { card: cardNumEl },
      });

      return confirmResult?.paymentIntent?.id || "";
    },
    [cardNumEl, stripe]
  );

  useEffect(() => {
    if (paymentConfirmToken) {
      confirmCardPayment(paymentConfirmToken)
        .catch((err) => {
          setError(String(err));
          setTimeout(() => confirmCardPayment(paymentConfirmToken), 5000);
        })
        .then((paymentIntentId) => {
          if (paymentIntentId) {
            onPaymentConfirmed(paymentIntentId);
          }
        });
    }
  }, [confirmCardPayment, onPaymentConfirmed, paymentConfirmToken]);

  return (
    <Form<{ stripe?: string }>
      onSubmit={() => (onPaymentSubmit ? onPaymentSubmit() : {})}
      render={({ submitting, handleSubmit }) => (
        <form
          onSubmit={(e) => {
            e.preventDefault();
            handleSubmit();
          }}
        >
          <CardContainer>
            <CardHeader>
              <CardTitle>Payment methods</CardTitle>
            </CardHeader>

            <PaymentFormBody>
              <Field name="stripe" validate={validate}>
                {({ input, meta }) => (
                  <StripePayment input={input} meta={meta} />
                )}
              </Field>
              <PaymentDetailsContainer>
                <div>Price: {payDetails.price.toFixed(2)}</div>
                <div>Tax: {payDetails.tax.toFixed(2)}</div>
                <div>
                  Total (£): {(payDetails.price + payDetails.tax).toFixed(2)}
                </div>
              </PaymentDetailsContainer>
            </PaymentFormBody>
            {error && <ErrorContainer>{error}</ErrorContainer>}

            <Button
              type="submit"
              disabled={!!paymentConfirmToken || submitting || !formComplete}
              width={"100%"}
            >
              {(transactionInProgress || submitting) && (
                <SpinnerContainer>
                  <Spinner color={colors.white} size="md" />
                </SpinnerContainer>
              )}
              Pay
            </Button>
          </CardContainer>
        </form>
      )}
    ></Form>
  );
};

export default PaymentForm;
