import React, { FC, useCallback, useEffect, useMemo, useState } from "react";

import { EventEmitter } from "events";
import { withApollo, WithApolloClient } from "react-apollo";

import { offer_api_rto_RtoStateEnum } from "../../../../../globalTypes";
import Spinner from "../../../components/Spinner";
import { BankDetails } from "../../../models/account";
import { Address } from "../../../models/address";
import { Rto } from "../../../models/offer";
import { ProductCodesByType, PropertyProduct } from "../../../models/product";
import { Property } from "../../../models/property";
import { createAccountService } from "../../../service/account";
import { createInvoiceService } from "../../../service/invoice";
import { createOfferService } from "../../../service/offer";
import { createPropertyService } from "../../../service/property";
import {
  CardContainer,
  CardHeader,
  CardTitle,
  ErrorContainer,
  SpinnerContainer,
} from "../../../styles";
import { colors } from "../../../theme";
import AddressModal from "../AddressModal";
import BankDetailsModal from "../BankDetailsModal";
import PropertyProductsList from "./ProductsList";
import RentCollectionModal from "./RentCollectionModal";
import { createPropertyListService } from "./service";
import {
  HeaderButton,
  HeaderRight,
  NewPropertyLink,
  PropertiesContainer,
  PropertyHeader,
  PropertyItem,
  PropertyItemHeader,
  PropertyItemStatus,
} from "./styles";

type PropertiesListProps = WithApolloClient<{
  userId: string;
  onPropertiesChange: (properties: Array<Property>) => void;
  events: EventEmitter;
}>;

export enum PropertiesListEvent {
  REFRESH,
}

const PropertiesList: FC<PropertiesListProps> = ({
  client,
  userId,
  onPropertiesChange,
  events,
}) => {
  const [error, setError] = useState<String>("");
  const [rtoModal, setRtoModal] = useState<{
    visible: boolean;
    rentData?: Rto;
  }>({ visible: false });
  const [bankDetailsModal, setBankDetailsModal] = useState<{
    visible: boolean;
    bankDetails?: BankDetails;
  }>({ visible: false });
  const [newPropertyModal, setNewPropertyModalVisible] = useState<boolean>(
    false
  );
  const [loading, setLoading] = useState<boolean>(false);
  const [properties, setProperties] = useState<Property[]>([]);

  const accountService = useMemo(() => createAccountService(client), [client]);
  const offerService = useMemo(() => createOfferService(client), [client]);
  const propertyService = useMemo(() => createPropertyService(client), [
    client,
  ]);
  const invoiceService = useMemo(() => createInvoiceService(client), [client]);
  const service = useMemo(() => createPropertyListService(client), [client]);

  const restoreProductsSelection = useCallback(
    (newProps: Array<Property>, oldProps: Array<Property>) => {
      const oldPropsByIdMap = new Map(oldProps.map((p) => [p.id, p]));
      for (const newProp of newProps) {
        const oldProd = oldPropsByIdMap.get(newProp.id);
        if (!oldProd) {
          continue;
        }

        const selProds = oldProd.products?.filter((pd) => pd.selected) || [];
        const selProdsCodes = selProds.map((p) => p.code);
        const selProdsByCodeMap = new Map(selProds.map((p) => [p.code, p]));

        const newProds = new Array<PropertyProduct>();
        for (const prod of newProp.products || []) {
          const type = prod.type;
          const codes = ProductCodesByType.get(type) || [];
          const selCode = codes.find((code) => selProdsCodes.includes(code));
          const selProd = selProdsByCodeMap.get(selCode || "");
          if (selProd) {
            newProds.push({ ...selProd });
          } else {
            newProds.push(prod);
          }
        }

        newProp.products = newProds;
      }
    },
    []
  );

  const fetchProperties = useCallback(
    async (userId: string): Promise<Array<Property>> => {
      setError("");
      setLoading(true);

      let props: Array<Property> = [];
      try {
        props = await service.fetchProperties(userId);
        if (props.length) {
          const extrasProducts = await service.fetchExtrasProducts();

          let orders = await service.fetchOrders(userId);
          orders = orders.filter((ord) => ord.correlationType === "property");

          const activeRtos = await offerService.fetchRtos(userId, [
            offer_api_rto_RtoStateEnum.RTO_STATE_PENDING_SETUP,
            offer_api_rto_RtoStateEnum.RTO_STATE_WAITING_FOR_TENANT_CONFIRMATION,
            offer_api_rto_RtoStateEnum.RTO_STATE_WAITING_FOR_STANDING_ORDER_SETUP,
            offer_api_rto_RtoStateEnum.RTO_STATE_WAITING_FOR_PAYMENT,
            offer_api_rto_RtoStateEnum.RTO_STATE_ACTIVE,
          ]);

          const invoices = await invoiceService.getStandingOrderInvoices(
            userId
          );

          props = service.enrichProperties(
            props,
            extrasProducts,
            orders,
            activeRtos,
            invoices
          );
        }
      } catch (err) {
        setError(String(err));
      } finally {
        setLoading(false);
        setProperties([...props]);
      }

      return props;
    },
    [service, offerService, invoiceService]
  );

  const onCreateNewProperty = React.useCallback(
    async (addrUrl: string, address: Address): Promise<void> => {
      setError("");

      await service
        .createPropertyVersion(userId, addrUrl, address)
        .catch((err) => setError(String(err)))
        .then((verId) =>
          !verId ? setError("Error while creating listing.") : ""
        )
        .finally(() => setNewPropertyModalVisible(false));

      const oldProps = [...properties];
      const newProps = await fetchProperties(userId);
      restoreProductsSelection(newProps, oldProps);
      setProperties([...newProps]);
    },
    [service, fetchProperties, userId, properties, restoreProductsSelection]
  );

  const onCreateNewRto = React.useCallback(
    async (rto: Rto): Promise<void> => {
      setError("");

      try {
        rto.landlordId = userId;
        rto.id = await offerService.createRto(rto);
        await offerService.createRtoTenancy(rto);

        setError("");
        setRtoModal({ visible: false });

        const newProps = await fetchProperties(userId);
        setProperties([...newProps]);
      } catch (err) {
        setError(String(err));
      }
    },
    [offerService, fetchProperties, userId]
  );

  const onPropertyBankDetailsUpdate = React.useCallback(
    async (bankDetails: BankDetails): Promise<void> => {
      setError("");

      try {
        await propertyService.updateProperty({
          id: bankDetails.propertyId,
          bankDetails,
        } as Property);
        setError("");
        setBankDetailsModal({ visible: false });

        const newProps = await fetchProperties(userId);
        setProperties([...newProps]);
      } catch (err) {
        setError(String(err));
      }
    },
    [propertyService, fetchProperties, userId]
  );

  const onCopyBankDetailsFromProfile = React.useCallback(async (): Promise<
    void
  > => {
    setError("");

    try {
      const account = await accountService.getAccountById(userId);

      setError("");
      setBankDetailsModal({
        visible: true,
        bankDetails: {
          propertyId: bankDetailsModal.bankDetails?.propertyId,
          accountName: account.bankDetails?.accountName || "",
          personalReference: account.bankDetails?.personalReference || "",
          accountNumber: account.bankDetails?.accountNumber || "",
          sortCode: account.bankDetails?.sortCode || "",
        },
      });
    } catch (err) {
      setError(String(err));
    }
  }, [accountService, userId, bankDetailsModal]);

  const onProductsChange = React.useCallback(
    (prop: Property, prods: Array<PropertyProduct>): void => {
      const changedProp = properties.find((p) => p.id === prop.id);
      if (changedProp) {
        changedProp.products = prods;
        setProperties([...properties]);
        onPropertiesChange(properties);
      }
    },
    [properties, onPropertiesChange]
  );

  useEffect(() => {
    (async () => {
      await fetchProperties(userId);
    })();

    events.on(PropertiesListEvent.REFRESH.toString(), async () => {
      await fetchProperties(userId);
    });
  }, [fetchProperties, userId, events]);

  return (
    <>
      <CardContainer>
        <CardHeader>
          <CardTitle>Properties ({properties?.length || 0})</CardTitle>
          <NewPropertyLink onClick={() => setNewPropertyModalVisible(true)}>
            Add a new property
          </NewPropertyLink>
        </CardHeader>

        {loading && (
          <SpinnerContainer>
            <Spinner color={colors.mashPaleRed} size="md" />
          </SpinnerContainer>
        )}
        {error && <ErrorContainer> {error}</ErrorContainer>}

        {properties && !loading && (
          <PropertiesContainer>
            {properties.map((prop) => (
              <PropertyItem key={prop.id}>
                <PropertyHeader>
                  <PropertyItemStatus status={prop.status}>
                    {prop.status}
                  </PropertyItemStatus>

                  <HeaderRight>
                    <HeaderButton
                      active={!!prop.activeRto}
                      onClick={(_) =>
                        setRtoModal({
                          visible: true,
                          rentData: prop.activeRto
                            ? prop.activeRto
                            : ({ propertyId: prop.id } as Rto),
                        })
                      }
                    >
                      Let & Protect
                    </HeaderButton>
                    <HeaderButton
                      active={!!prop.bankDetails?.accountNumber}
                      onClick={(_) =>
                        setBankDetailsModal({
                          visible: true,
                          bankDetails: prop.bankDetails
                            ? prop.bankDetails
                            : ({ propertyId: prop.id } as BankDetails),
                        })
                      }
                    >
                      {prop.bankDetails?.accountNumber
                        ? "Bank Details"
                        : "NO Bank Details"}
                    </HeaderButton>
                  </HeaderRight>
                </PropertyHeader>
                <PropertyItemHeader>{prop.addressLine}</PropertyItemHeader>
                <PropertyProductsList
                  propertyId={prop.id}
                  products={prop.products || []}
                  onProductsChange={(prods) => onProductsChange(prop, prods)}
                ></PropertyProductsList>
              </PropertyItem>
            ))}
          </PropertiesContainer>
        )}
      </CardContainer>

      {newPropertyModal && (
        <AddressModal
          title={"Create a property"}
          onSubmit={onCreateNewProperty}
          onCancel={() => setNewPropertyModalVisible(false)}
        ></AddressModal>
      )}

      {rtoModal.visible && (
        <RentCollectionModal
          title="Transfer Rent Collection"
          rentData={rtoModal.rentData || ({} as Rto)}
          onSubmit={onCreateNewRto}
          onCancel={() => {
            setRtoModal({ visible: false });
            setError("");
          }}
          errors={[error]}
        ></RentCollectionModal>
      )}

      {bankDetailsModal.visible && (
        <BankDetailsModal
          title="Bank Details"
          bankDetails={bankDetailsModal.bankDetails || ({} as BankDetails)}
          onSubmit={onPropertyBankDetailsUpdate}
          onCopyFromProfile={onCopyBankDetailsFromProfile}
          onCancel={() => {
            setBankDetailsModal({ visible: false });
            setError("");
          }}
          errors={[error]}
        ></BankDetailsModal>
      )}
    </>
  );
};

export default withApollo<PropertiesListProps>(PropertiesList);
