import React, { FC, useCallback, useEffect, useState } from "react";

import styled from "@emotion/styled";
import AntdButton from "antd-button-color";
import { useMutation, useQuery } from "react-apollo";
import { Field, Form } from "react-final-form";
import { OnChange } from "react-final-form-listeners";

import {
  common_api_generic_query_QueryFilterOperatorEnum as OperatorEnum,
  invoice_api_InvoiceStatusEnum,
  invoice_api_TransactionTypeEnum,
} from "../../globalTypes";
import { Invoice_Api_PaymentStatusEnum } from "../../graphql-schema-types.generated";
import Spinner from "../../ui/atoms/Spinner";
import { Td, Tr } from "../../ui/atoms/Table";
import Dropdown from "../../ui/molecules/Dropdown";
import { formatDate } from "../../util/date";
import {
  AddPaymentTransactionMatch,
  AddPaymentTransactionMatchVariables,
} from "../payments/graphql/mutations/__generated__/AddPaymentTransactionMatch";
import ADD_TRANSACTION_MATCH from "../payments/graphql/mutations/AddPaymentTransactionMatch.graphql";
import {
  PaymentsGetAllInvoices,
  PaymentsGetAllInvoicesVariables,
} from "../payments/graphql/queries/__generated__/PaymentsGetAllInvoices";
import FETCH_INVOICES from "../payments/graphql/queries/PaymentsGetAllInvoices.graphql";
import {
  PaymentsGetInvoiceQuery,
  PaymentsGetInvoiceQueryVariables,
} from "../payments/graphql/queries/PaymentsGetInvoice.generated";
import GET_INVOICE from "../payments/graphql/queries/PaymentsGetInvoice.graphql";
import {
  AVAILABLE_CORRELATION_TARGETS,
  CORRELATION_TARGET_LABEL,
} from "./constants";
import FindAddress from "./FindAddress";
import { GetTransactions_admin_allTransactions_transactions_matches } from "./graphql/__generated__/GetTransactions";
import { useTransactionMatchPropertyAddress } from "./hooks";
import { getEarliestItemFromArray, isEarlier } from "./utils";

type Props = {
  amount: number;
  description: string;
  transaction_id: string;
  ivr: string;
  transactionMatches:
    | readonly GetTransactions_admin_allTransactions_transactions_matches[]
    | null;
  transaction_date: string;
  type: string;
};

type FormValues = {
  location: string;
  invoice_id: string | undefined;
  property_id: string;
  payment_id: string;
};

type oneOption = {
  label: string;
  value: string;
};

const ListItem: FC<Props> = ({
  amount,
  description,
  transaction_id,
  ivr,
  transactionMatches,
  transaction_date,
  type,
}) => {
  const [paymentOptions, setPaymentOptions] = useState<oneOption[]>([]);
  const [buttonMerge, setButtonMerge] = useState<boolean>(false);
  const [createPayment, { loading: createPaymentLoading }] = useMutation<
    AddPaymentTransactionMatch,
    AddPaymentTransactionMatchVariables
  >(ADD_TRANSACTION_MATCH);

  const [propertyInput, setPropertyInput] = useState<FormValues>({
    location: "",
    property_id: "",
    invoice_id: "",
    payment_id: "",
  });
  const admin_id = window.localStorage.getItem("adminId") ?? "";

  const {
    address: transactionMatchPropertyAddress,
    cityPostcodeAndCountry: transactionMatchPropertyCityPostcodeAndCountry,
  } = useTransactionMatchPropertyAddress(transactionMatches);

  const withPrefilledPropertyAddress =
    Boolean(transactionMatchPropertyAddress) ||
    Boolean(transactionMatchPropertyCityPostcodeAndCountry);

  const {
    data: dataInvoices,
    loading: invoicesLoading,
    networkStatus: invoicesStatus,
  } = useQuery<PaymentsGetAllInvoices, PaymentsGetAllInvoicesVariables>(
    FETCH_INVOICES,
    {
      variables: {
        pagination: { page: 1, perPage: 400 },
        filter: [
          {
            fields: ["status"],
            not: true,
            operator: OperatorEnum.ONEOF,
            value: [
              invoice_api_InvoiceStatusEnum.INVOICE_FULFILLED,
              invoice_api_InvoiceStatusEnum.INVOICE_CANCELED,
            ],
          },
          {
            fields: ["correlation_target"],
            operator: OperatorEnum.CONTAINS_ANY,
            value: AVAILABLE_CORRELATION_TARGETS,
          },
          {
            fields: ["property"],
            operator: OperatorEnum.EQUALS,
            value: [propertyInput.property_id],
          },
        ],
      },
      skip: propertyInput.location === "" || propertyInput.property_id === "",
    }
  );

  useEffect(() => {
    setPaymentOptions([]);
    const newPaymentOptions = [] as oneOption[];

    const allInvoices = [...(dataInvoices?.admin?.allInvoices?.invoices ?? [])];

    const invoiceByCorrelationTarget = allInvoices.reduce((group, invoice) => {
      const { correlation_target } = invoice;
      if (AVAILABLE_CORRELATION_TARGETS.includes(correlation_target)) {
        group[correlation_target] =
          !group[correlation_target] ||
          isEarlier(invoice.updated_at, group[correlation_target].updated_at)
            ? invoice
            : group[correlation_target];
      }
      return group;
    }, {});

    Object.keys(invoiceByCorrelationTarget).forEach((key) => {
      const selected = invoiceByCorrelationTarget[key];
      const option = {
        label: `${CORRELATION_TARGET_LABEL[selected.correlation_target]}: ${
          selected.amount
        }`,
        value: selected.invoice_id,
      };
      newPaymentOptions.push(option);
    });

    setPaymentOptions(newPaymentOptions);
  }, [dataInvoices]);

  const { data: dataInvoicePayments } = useQuery<
    PaymentsGetInvoiceQuery,
    PaymentsGetInvoiceQueryVariables
  >(GET_INVOICE, {
    variables: {
      invoice_id: propertyInput.invoice_id,
    },
    skip: propertyInput.invoice_id === "",
  });

  useEffect(() => {
    if (dataInvoicePayments !== undefined) {
      const filteredPayments = dataInvoicePayments.admin?.Invoice?.payments?.filter(
        ({ status }) => status === Invoice_Api_PaymentStatusEnum.PaymentPending
      );
      if (filteredPayments && filteredPayments.length > 0) {
        const selectedPayment = getEarliestItemFromArray(filteredPayments);
        setPropertyInput((state) => ({
          ...state,
          payment_id: selectedPayment?.id ?? "",
        }));
      }
    }
  }, [dataInvoicePayments]);

  const handleUpdateStatus = async (transaction_id: string) => {
    await createPayment({
      variables: {
        transaction_id,
        comment: "custom matching",
        admin_id,
      },
      refetchQueries: ["GetTransactions"],
    });
  };

  const handleMergeButtonClick = useCallback(
    () => setButtonMerge((state) => !state),
    []
  );

  return (
    <>
      <Tr>
        <Td colSpan={3}>
          <span>
            {withPrefilledPropertyAddress ? (
              <>
                {transactionMatchPropertyAddress}
                {transactionMatchPropertyCityPostcodeAndCountry && (
                  <div>{transactionMatchPropertyCityPostcodeAndCountry}</div>
                )}
              </>
            ) : (
              description
            )}
          </span>
        </Td>
        <Td colSpan={2}>
          <span>£{amount}</span>
        </Td>
        <Td colSpan={2}>
          <span>{ivr}</span>
        </Td>
        <Td colSpan={2}>
          <span>{formatDate(transaction_date)}</span>
        </Td>
        <Td colSpan={1}>
          <span>
            {type === invoice_api_TransactionTypeEnum.TRANSACTION_TYPE_DEBIT
              ? "Debit"
              : "Credit"}
          </span>
        </Td>
        <Td colSpan={2}>
          {!withPrefilledPropertyAddress && (
            <AntdButton block onClick={handleMergeButtonClick} type="link">
              Merge
            </AntdButton>
          )}
        </Td>
      </Tr>
      {!withPrefilledPropertyAddress && buttonMerge && (
        <Form<FormValues>
          onSubmit={() => {
            if (propertyInput.invoice_id && transaction_id) {
              handleUpdateStatus(transaction_id);
            }
          }}
        >
          {() => {
            return (
              <Tr>
                <Td colSpan={5}>
                  <FindAddress setPropertyInput={setPropertyInput} />
                </Td>
                <Td colSpan={2}></Td>
                <Td colSpan={3}>
                  <div>
                    {invoicesLoading ? (
                      <Loader>
                        <Spinner />
                      </Loader>
                    ) : paymentOptions.length > 0 ? (
                      <Field
                        name="invoice_id"
                        render={({ input }) => (
                          <Dropdown
                            fullWidth
                            input={input}
                            options={paymentOptions}
                            placeholder="Payment date"
                          />
                        )}
                      />
                    ) : Boolean(invoicesStatus) ? (
                      <span>no appropriate invoices</span>
                    ) : null}

                    <OnChange name="invoice_id">
                      {(value: any) => {
                        if (value) {
                          setPropertyInput((state) => ({
                            ...state,
                            invoice_id: value.value,
                          }));
                        }
                      }}
                    </OnChange>
                  </div>
                </Td>

                <Td colSpan={2}>
                  <AntdButton
                    block
                    disabled={!(propertyInput.invoice_id && transaction_id)}
                    htmlType="button"
                    onClick={() => {
                      handleUpdateStatus(transaction_id);
                      setButtonMerge(false);
                    }}
                    type="primary"
                  >
                    {createPaymentLoading ? "loading" : "Complete merge"}
                  </AntdButton>
                </Td>
              </Tr>
            );
          }}
        </Form>
      )}
    </>
  );
};

const Loader = styled.div`
  width: 16px;
  height: 16px;
  display: flex;
  align-items: center;
  justify-content: center;
`;

export default ListItem;
